(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(require('codemirror')) :
    typeof define === 'function' && define.amd ? define(['codemirror'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.CodeMirror));
}(this, (function (CodeMirror) { 'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var CodeMirror__default = /*#__PURE__*/_interopDefaultLegacy(CodeMirror);

    function opt(ofRule) {
        return { ofRule };
    }
    function list(ofRule, separator) {
        return { ofRule, isList: true, separator };
    }
    function butNot(rule, exclusions) {
        const ruleMatch = rule.match;
        rule.match = token => {
            let check = false;
            if (ruleMatch) {
                check = ruleMatch(token);
            }
            return (check &&
                exclusions.every(exclusion => exclusion.match && !exclusion.match(token)));
        };
        return rule;
    }
    function t(kind, style) {
        return { style, match: (token) => token.kind === kind };
    }
    function p(value, style) {
        return {
            style: style || 'punctuation',
            match: (token) => token.kind === 'Punctuation' && token.value === value,
        };
    }

    const isIgnored = (ch) => ch === ' ' ||
        ch === '\t' ||
        ch === ',' ||
        ch === '\n' ||
        ch === '\r' ||
        ch === '\uFEFF' ||
        ch === '\u00A0';
    const LexRules = {
        Name: /^[_A-Za-z][_0-9A-Za-z]*/,
        Punctuation: /^(?:!|\$|\(|\)|\.\.\.|:|=|&|@|\[|]|\{|\||\})/,
        Number: /^-?(?:0|(?:[1-9][0-9]*))(?:\.[0-9]*)?(?:[eE][+-]?[0-9]+)?/,
        String: /^(?:"""(?:\\"""|[^"]|"[^"]|""[^"])*(?:""")?|"(?:[^"\\]|\\(?:"|\/|\\|b|f|n|r|t|u[0-9a-fA-F]{4}))*"?)/,
        Comment: /^#.*/,
    };
    const ParseRules = {
        Document: [list('Definition')],
        Definition(token) {
            switch (token.value) {
                case '{':
                    return 'ShortQuery';
                case 'query':
                    return 'Query';
                case 'mutation':
                    return 'Mutation';
                case 'subscription':
                    return 'Subscription';
                case 'fragment':
                    return 'FragmentDefinition';
                case 'schema':
                    return 'SchemaDef';
                case 'scalar':
                    return 'ScalarDef';
                case 'type':
                    return 'ObjectTypeDef';
                case 'interface':
                    return 'InterfaceDef';
                case 'union':
                    return 'UnionDef';
                case 'enum':
                    return 'EnumDef';
                case 'input':
                    return 'InputDef';
                case 'extend':
                    return 'ExtendDef';
                case 'directive':
                    return 'DirectiveDef';
            }
        },
        ShortQuery: ['SelectionSet'],
        Query: [
            word('query'),
            opt(name('def')),
            opt('VariableDefinitions'),
            list('Directive'),
            'SelectionSet',
        ],
        Mutation: [
            word('mutation'),
            opt(name('def')),
            opt('VariableDefinitions'),
            list('Directive'),
            'SelectionSet',
        ],
        Subscription: [
            word('subscription'),
            opt(name('def')),
            opt('VariableDefinitions'),
            list('Directive'),
            'SelectionSet',
        ],
        VariableDefinitions: [p('('), list('VariableDefinition'), p(')')],
        VariableDefinition: ['Variable', p(':'), 'Type', opt('DefaultValue')],
        Variable: [p('$', 'variable'), name('variable')],
        DefaultValue: [p('='), 'Value'],
        SelectionSet: [p('{'), list('Selection'), p('}')],
        Selection(token, stream) {
            return token.value === '...'
                ? stream.match(/[\s\u00a0,]*(on\b|@|{)/, false)
                    ? 'InlineFragment'
                    : 'FragmentSpread'
                : stream.match(/[\s\u00a0,]*:/, false)
                    ? 'AliasedField'
                    : 'Field';
        },
        AliasedField: [
            name('property'),
            p(':'),
            name('qualifier'),
            opt('Arguments'),
            list('Directive'),
            opt('SelectionSet'),
        ],
        Field: [
            name('property'),
            opt('Arguments'),
            list('Directive'),
            opt('SelectionSet'),
        ],
        Arguments: [p('('), list('Argument'), p(')')],
        Argument: [name('attribute'), p(':'), 'Value'],
        FragmentSpread: [p('...'), name('def'), list('Directive')],
        InlineFragment: [
            p('...'),
            opt('TypeCondition'),
            list('Directive'),
            'SelectionSet',
        ],
        FragmentDefinition: [
            word('fragment'),
            opt(butNot(name('def'), [word('on')])),
            'TypeCondition',
            list('Directive'),
            'SelectionSet',
        ],
        TypeCondition: [word('on'), 'NamedType'],
        Value(token) {
            switch (token.kind) {
                case 'Number':
                    return 'NumberValue';
                case 'String':
                    return 'StringValue';
                case 'Punctuation':
                    switch (token.value) {
                        case '[':
                            return 'ListValue';
                        case '{':
                            return 'ObjectValue';
                        case '$':
                            return 'Variable';
                        case '&':
                            return 'NamedType';
                    }
                    return null;
                case 'Name':
                    switch (token.value) {
                        case 'true':
                        case 'false':
                            return 'BooleanValue';
                    }
                    if (token.value === 'null') {
                        return 'NullValue';
                    }
                    return 'EnumValue';
            }
        },
        NumberValue: [t('Number', 'number')],
        StringValue: [t('String', 'string')],
        BooleanValue: [t('Name', 'builtin')],
        NullValue: [t('Name', 'keyword')],
        EnumValue: [name('string-2')],
        ListValue: [p('['), list('Value'), p(']')],
        ObjectValue: [p('{'), list('ObjectField'), p('}')],
        ObjectField: [name('attribute'), p(':'), 'Value'],
        Type(token) {
            return token.value === '[' ? 'ListType' : 'NonNullType';
        },
        ListType: [p('['), 'Type', p(']'), opt(p('!'))],
        NonNullType: ['NamedType', opt(p('!'))],
        NamedType: [type('atom')],
        Directive: [p('@', 'meta'), name('meta'), opt('Arguments')],
        DirectiveDef: [
            word('directive'),
            p('@', 'meta'),
            name('meta'),
            opt('ArgumentsDef'),
            word('on'),
            list('DirectiveLocation', p('|')),
        ],
        InterfaceDef: [
            word('interface'),
            name('atom'),
            opt('Implements'),
            list('Directive'),
            p('{'),
            list('FieldDef'),
            p('}'),
        ],
        Implements: [word('implements'), list('NamedType', p('&'))],
        DirectiveLocation: [name('string-2')],
        SchemaDef: [
            word('schema'),
            list('Directive'),
            p('{'),
            list('OperationTypeDef'),
            p('}'),
        ],
        OperationTypeDef: [name('keyword'), p(':'), name('atom')],
        ScalarDef: [word('scalar'), name('atom'), list('Directive')],
        ObjectTypeDef: [
            word('type'),
            name('atom'),
            opt('Implements'),
            list('Directive'),
            p('{'),
            list('FieldDef'),
            p('}'),
        ],
        FieldDef: [
            name('property'),
            opt('ArgumentsDef'),
            p(':'),
            'Type',
            list('Directive'),
        ],
        ArgumentsDef: [p('('), list('InputValueDef'), p(')')],
        InputValueDef: [
            name('attribute'),
            p(':'),
            'Type',
            opt('DefaultValue'),
            list('Directive'),
        ],
        UnionDef: [
            word('union'),
            name('atom'),
            list('Directive'),
            p('='),
            list('UnionMember', p('|')),
        ],
        UnionMember: ['NamedType'],
        EnumDef: [
            word('enum'),
            name('atom'),
            list('Directive'),
            p('{'),
            list('EnumValueDef'),
            p('}'),
        ],
        EnumValueDef: [name('string-2'), list('Directive')],
        InputDef: [
            word('input'),
            name('atom'),
            list('Directive'),
            p('{'),
            list('InputValueDef'),
            p('}'),
        ],
        ExtendDef: [word('extend'), 'ObjectTypeDef'],
    };
    function word(value) {
        return {
            style: 'keyword',
            match: (token) => token.kind === 'Name' && token.value === value,
        };
    }
    function name(style) {
        return {
            style,
            match: (token) => token.kind === 'Name',
            update(state, token) {
                state.name = token.value;
            },
        };
    }
    function type(style) {
        return {
            style,
            match: (token) => token.kind === 'Name',
            update(state, token) {
                if (state.prevState && state.prevState.prevState) {
                    state.name = token.value;
                    state.prevState.prevState.type = token.value;
                }
            },
        };
    }

    function onlineParser(options = {
        eatWhitespace: stream => stream.eatWhile(isIgnored),
        lexRules: LexRules,
        parseRules: ParseRules,
        editorConfig: {},
    }) {
        return {
            startState() {
                const initialState = {
                    level: 0,
                    step: 0,
                    name: null,
                    kind: null,
                    type: null,
                    rule: null,
                    needsSeperator: false,
                    prevState: null,
                };
                pushRule(options.parseRules, initialState, 'Document');
                return initialState;
            },
            token(stream, state) {
                return getToken(stream, state, options);
            },
        };
    }
    function getToken(stream, state, options) {
        const { lexRules, parseRules, eatWhitespace, editorConfig } = options;
        if (state.rule && state.rule.length === 0) {
            popRule(state);
        }
        else if (state.needsAdvance) {
            state.needsAdvance = false;
            advanceRule(state, true);
        }
        if (stream.sol()) {
            const tabSize = (editorConfig && editorConfig.tabSize) || 2;
            state.indentLevel = Math.floor(stream.indentation() / tabSize);
        }
        if (eatWhitespace(stream)) {
            return 'ws';
        }
        const token = lex(lexRules, stream);
        if (!token) {
            const matchedSomething = stream.match(/\S+/);
            if (!matchedSomething) {
                stream.match(/\s/);
            }
            pushRule(SpecialParseRules, state, 'Invalid');
            return 'invalidchar';
        }
        if (token.kind === 'Comment') {
            pushRule(SpecialParseRules, state, 'Comment');
            return 'comment';
        }
        const backupState = assign({}, state);
        if (token.kind === 'Punctuation') {
            if (/^[{([]/.test(token.value)) {
                if (state.indentLevel !== undefined) {
                    state.levels = (state.levels || []).concat(state.indentLevel + 1);
                }
            }
            else if (/^[})\]]/.test(token.value)) {
                const levels = (state.levels = (state.levels || []).slice(0, -1));
                if (state.indentLevel) {
                    if (levels.length > 0 &&
                        levels[levels.length - 1] < state.indentLevel) {
                        state.indentLevel = levels[levels.length - 1];
                    }
                }
            }
        }
        while (state.rule) {
            let expected = typeof state.rule === 'function'
                ? state.step === 0
                    ? state.rule(token, stream)
                    : null
                : state.rule[state.step];
            if (state.needsSeperator) {
                expected = expected && (expected === null || expected === void 0 ? void 0 : expected.separator);
            }
            if (expected) {
                if (expected.ofRule) {
                    expected = expected.ofRule;
                }
                if (typeof expected === 'string') {
                    pushRule(parseRules, state, expected);
                    continue;
                }
                if (expected.match && expected.match(token)) {
                    if (expected.update) {
                        expected.update(state, token);
                    }
                    if (token.kind === 'Punctuation') {
                        advanceRule(state, true);
                    }
                    else {
                        state.needsAdvance = true;
                    }
                    return expected.style;
                }
            }
            unsuccessful(state);
        }
        assign(state, backupState);
        pushRule(SpecialParseRules, state, 'Invalid');
        return 'invalidchar';
    }
    function assign(to, from) {
        const keys = Object.keys(from);
        for (let i = 0; i < keys.length; i++) {
            to[keys[i]] = from[keys[i]];
        }
        return to;
    }
    const SpecialParseRules = {
        Invalid: [],
        Comment: [],
    };
    function pushRule(rules, state, ruleKind) {
        if (!rules[ruleKind]) {
            throw new TypeError('Unknown rule: ' + ruleKind);
        }
        state.prevState = Object.assign({}, state);
        state.kind = ruleKind;
        state.name = null;
        state.type = null;
        state.rule = rules[ruleKind];
        state.step = 0;
        state.needsSeperator = false;
    }
    function popRule(state) {
        if (!state.prevState) {
            return;
        }
        state.kind = state.prevState.kind;
        state.name = state.prevState.name;
        state.type = state.prevState.type;
        state.rule = state.prevState.rule;
        state.step = state.prevState.step;
        state.needsSeperator = state.prevState.needsSeperator;
        state.prevState = state.prevState.prevState;
    }
    function advanceRule(state, successful) {
        if (isList(state) && state.rule) {
            const step = state.rule[state.step];
            if (step.separator) {
                const separator = step.separator;
                state.needsSeperator = !state.needsSeperator;
                if (!state.needsSeperator && separator.ofRule) {
                    return;
                }
            }
            if (successful) {
                return;
            }
        }
        state.needsSeperator = false;
        state.step++;
        while (state.rule &&
            !(Array.isArray(state.rule) && state.step < state.rule.length)) {
            popRule(state);
            if (state.rule) {
                if (isList(state)) {
                    if (state.rule && state.rule[state.step].separator) {
                        state.needsSeperator = !state.needsSeperator;
                    }
                }
                else {
                    state.needsSeperator = false;
                    state.step++;
                }
            }
        }
    }
    function isList(state) {
        const step = Array.isArray(state.rule) &&
            typeof state.rule[state.step] !== 'string' &&
            state.rule[state.step];
        return step && step.isList;
    }
    function unsuccessful(state) {
        while (state.rule &&
            !(Array.isArray(state.rule) && state.rule[state.step].ofRule)) {
            popRule(state);
        }
        if (state.rule) {
            advanceRule(state, false);
        }
    }
    function lex(lexRules, stream) {
        const kinds = Object.keys(lexRules);
        for (let i = 0; i < kinds.length; i++) {
            const match = stream.match(lexRules[kinds[i]]);
            if (match && match instanceof Array) {
                return { kind: kinds[i], value: match[0] };
            }
        }
    }

    /**
     * The set of allowed kind values for AST nodes.
     */
    var Kind = Object.freeze({
      // Name
      NAME: 'Name',
      // Document
      DOCUMENT: 'Document',
      OPERATION_DEFINITION: 'OperationDefinition',
      VARIABLE_DEFINITION: 'VariableDefinition',
      SELECTION_SET: 'SelectionSet',
      FIELD: 'Field',
      ARGUMENT: 'Argument',
      // Fragments
      FRAGMENT_SPREAD: 'FragmentSpread',
      INLINE_FRAGMENT: 'InlineFragment',
      FRAGMENT_DEFINITION: 'FragmentDefinition',
      // Values
      VARIABLE: 'Variable',
      INT: 'IntValue',
      FLOAT: 'FloatValue',
      STRING: 'StringValue',
      BOOLEAN: 'BooleanValue',
      NULL: 'NullValue',
      ENUM: 'EnumValue',
      LIST: 'ListValue',
      OBJECT: 'ObjectValue',
      OBJECT_FIELD: 'ObjectField',
      // Directives
      DIRECTIVE: 'Directive',
      // Types
      NAMED_TYPE: 'NamedType',
      LIST_TYPE: 'ListType',
      NON_NULL_TYPE: 'NonNullType',
      // Type System Definitions
      SCHEMA_DEFINITION: 'SchemaDefinition',
      OPERATION_TYPE_DEFINITION: 'OperationTypeDefinition',
      // Type Definitions
      SCALAR_TYPE_DEFINITION: 'ScalarTypeDefinition',
      OBJECT_TYPE_DEFINITION: 'ObjectTypeDefinition',
      FIELD_DEFINITION: 'FieldDefinition',
      INPUT_VALUE_DEFINITION: 'InputValueDefinition',
      INTERFACE_TYPE_DEFINITION: 'InterfaceTypeDefinition',
      UNION_TYPE_DEFINITION: 'UnionTypeDefinition',
      ENUM_TYPE_DEFINITION: 'EnumTypeDefinition',
      ENUM_VALUE_DEFINITION: 'EnumValueDefinition',
      INPUT_OBJECT_TYPE_DEFINITION: 'InputObjectTypeDefinition',
      // Directive Definitions
      DIRECTIVE_DEFINITION: 'DirectiveDefinition',
      // Type System Extensions
      SCHEMA_EXTENSION: 'SchemaExtension',
      // Type Extensions
      SCALAR_TYPE_EXTENSION: 'ScalarTypeExtension',
      OBJECT_TYPE_EXTENSION: 'ObjectTypeExtension',
      INTERFACE_TYPE_EXTENSION: 'InterfaceTypeExtension',
      UNION_TYPE_EXTENSION: 'UnionTypeExtension',
      ENUM_TYPE_EXTENSION: 'EnumTypeExtension',
      INPUT_OBJECT_TYPE_EXTENSION: 'InputObjectTypeExtension'
    });
    /**
     * The enum type representing the possible kind values of AST nodes.
     */

    const AdditionalRuleKinds = {
        ALIASED_FIELD: 'AliasedField',
        ARGUMENTS: 'Arguments',
        SHORT_QUERY: 'ShortQuery',
        QUERY: 'Query',
        MUTATION: 'Mutation',
        SUBSCRIPTION: 'Subscription',
        TYPE_CONDITION: 'TypeCondition',
        INVALID: 'Invalid',
        COMMENT: 'Comment',
        SCHEMA_DEF: 'SchemaDef',
        SCALAR_DEF: 'ScalarDef',
        OBJECT_TYPE_DEF: 'ObjectTypeDef',
        OBJECT_VALUE: 'ObjectValue',
        LIST_VALUE: 'ListValue',
        INTERFACE_DEF: 'InterfaceDef',
        UNION_DEF: 'UnionDef',
        ENUM_DEF: 'EnumDef',
        ENUM_VALUE: 'EnumValue',
        FIELD_DEF: 'FieldDef',
        INPUT_DEF: 'InputDef',
        INPUT_VALUE_DEF: 'InputValueDef',
        ARGUMENTS_DEF: 'ArgumentsDef',
        EXTEND_DEF: 'ExtendDef',
        DIRECTIVE_DEF: 'DirectiveDef',
        IMPLEMENTS: 'Implements',
        VARIABLE_DEFINITIONS: 'VariableDefinitions',
        TYPE: 'Type',
    };
    Object.assign(Object.assign({}, Kind), AdditionalRuleKinds);

    CodeMirror__default['default'].defineMode('graphql', config => {
        const parser = onlineParser({
            eatWhitespace: stream => stream.eatWhile(isIgnored),
            lexRules: LexRules,
            parseRules: ParseRules,
            editorConfig: { tabSize: config.tabSize },
        });
        return {
            config,
            startState: parser.startState,
            token: parser.token,
            indent,
            electricInput: /^\s*[})\]]/,
            fold: 'brace',
            lineComment: '#',
            closeBrackets: {
                pairs: '()[]{}""',
                explode: '()[]{}',
            },
        };
    });
    function indent(state, textAfter) {
        var _a, _b;
        const levels = state.levels;
        const level = !levels || levels.length === 0
            ? state.indentLevel
            : levels[levels.length - 1] -
                (((_a = this.electricInput) === null || _a === void 0 ? void 0 : _a.test(textAfter)) ? 1 : 0);
        return (level || 0) * (((_b = this.config) === null || _b === void 0 ? void 0 : _b.indentUnit) || 0);
    }

    CodeMirror__default['default'].defineMIME('application/graphql', 'graphql');

})));
